@@ -1,11 +1,33 @@
 expectile.restricted <-
-function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1) 
+function (formula, data = NULL, smooth = c("schall", "acv", "none"), 
+    lambda = 0.1, expectiles = NA, density = FALSE) 
 {
     smooth = match.arg(smooth)
-    pp <- c(0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 0.8, 0.9, 0.95, 
-        0.98, 0.99)
+    if (density) {
+        pp <- seq(0.01, 0.99, by = 0.01)
+        pp.plot <- c(1, 2, 5, 10, 20, 50, 80, 90, 95, 98, 99)
+        row.grid = 3
+        col.grid = 4
+    }
+    else if (any(is.na(expectiles)) || !is.vector(expectiles) || 
+        any(expectiles > 1) || any(expectiles < 0)) {
+        pp <- c(0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 0.8, 0.9, 0.95, 
+            0.98, 0.99)
+        pp.plot <- 1:length(pp)
+        row.grid = 3
+        col.grid = 4
+    }
+    else {
+        pp <- expectiles
+        pp.plot <- 1:length(pp)
+        row.grid = floor(sqrt(length(pp)))
+        col.grid = ceiling(sqrt(length(pp)))
+        if (length(pp) > row.grid * col.grid) 
+            row.grid = row.grid + 1
+    }
     np <- length(pp)
-    yy = eval(parse(text = formula[2]))
+    np.plot <- length(pp.plot)
+    yy = eval(parse(text = formula[2]), envir = data, enclos = .GlobalEnv)
     m = length(yy)
     design = list()
     x = list()
@@ -15,11 +37,24 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
     nb = vector()
     krig.phi = list()
     for (i in 1:length(labels(terms(formula)))) {
-        design[[i]] = eval(parse(text = labels(terms(formula))[i]))
+        types[[i]] = strsplit(labels(terms(formula))[i], "(", 
+            fixed = TRUE)[[1]][1]
+        if (types[[i]] == labels(terms(formula))[i]) {
+            design[[i]] = base(matrix(eval(parse(text = labels(terms(formula))[i]), 
+                envir = data, enclos = .GlobalEnv), nrow = m), 
+                "parametric")
+            formula = eval(substitute(update(formula, . ~ variable2 + 
+                . - variable1), list(variable1 = as.name(types[[i]]), 
+                variable2 = as.name(paste("base(", types[[i]], 
+                  ",'parametric')", sep = "")))))
+            types[[i]] = "parametric"
+        }
+        else design[[i]] = eval(parse(text = labels(terms(formula))[i]), 
+            envir = data, enclos = .GlobalEnv)
     }
     nterms = length(design)
     B = design[[1]][[1]]
-    DD = design[[1]][[2]]
+    DD = as.matrix(design[[1]][[2]])
     x[[1]] = design[[1]][[3]]
     types[[1]] = design[[1]][[4]]
     bnd[[1]] = design[[1]][[5]]
@@ -29,6 +64,7 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
     if (length(labels(terms(formula))) > 1) 
         for (i in 2:length(labels(terms(formula)))) {
             B = cbind(B, design[[i]][[1]])
+            design[[i]][[2]] = as.matrix(design[[i]][[2]])
             DD = rbind(cbind(DD, matrix(0, nrow = dim(DD)[1], 
                 ncol = dim(design[[i]][[2]])[1])), cbind(matrix(0, 
                 nrow = dim(design[[i]][[2]])[1], ncol = dim(DD)[2]), 
@@ -45,7 +81,6 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
     lala <- matrix(lambda, nrow = nterms, ncol = 2, dimnames = list(1:nterms, 
         c("mean", "residual")))
     vector.a.ma.schall <- matrix(NA, nrow = sum(nb) + 1, ncol = np)
-    med = which(pp == 0.5)
     penalty <- lambda
     if (smooth == "schall") {
         dc = 1
@@ -55,14 +90,15 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
         while (dc >= 0.01 && it < 100) {
             aa <- asyregpen.lsfit(yy, B, 0.5, lala[, 1], DD, 
                 nb)
-            vector.a.ma.schall[, med] <- aa$a
+            mean.coefficients <- aa$a
             sig2 <- vector()
             tau2 <- vector()
             l0 <- lala[, 1]
             for (i in 1:nterms) {
                 partbasis = (sum(nb[0:(i - 1)]) + 1):(sum(nb[0:i]))
-                partB = B[, -1][, partbasis]
-                partDD = DD[, -1][-1, ][partbasis, partbasis]
+                partB = B[, -1][, partbasis, drop = FALSE]
+                partDD = DD[, -1][-1, ][partbasis, partbasis, 
+                  drop = FALSE]
                 partaa = aa$a[-1][partbasis]
                 partw = aa$weight[-1][partbasis]
                 v <- partDD %*% partaa
@@ -86,30 +122,32 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
             DD = DD, nb = nb, ndigit = 8, iterlim = 50, gradtol = 1e-04)
         aa <- asyregpen.lsfit(yy, B, 0.5, abs(acv.min$estimate), 
             DD, nb)
-        vector.a.ma.schall[, med] <- aa$a
+        mean.coefficients <- aa$a
         lala[, 1] <- abs(acv.min$estimate)
     }
     else {
         if (length(penalty) < length(nb)) 
             penalty = rep(penalty[1], length(nb))
         aa <- asyregpen.lsfit(yy, B, 0.5, penalty, DD, nb)
-        vector.a.ma.schall[, med] <- aa$a
+        mean.coefficients <- aa$a
     }
-    residuals = yy - B %*% vector.a.ma.schall[, med]
+    residuals = yy - B %*% mean.coefficients
     gg = asyregpen.lsfit(abs(residuals), B, 0.5, lala[, 1], DD, 
         nb)
-    for (q in 1:np[-med]) {
+    cc = NULL
+    for (q in 1:np) {
         ca = asyregpen.lsfit(residuals, gg$fitted, pp[q], NULL, 
             matrix(0, nrow = 1, ncol = 1))
-        vector.a.ma.schall[, q] = vector.a.ma.schall[, med] + 
-            ca$a * gg$a
+        cc[q] = ca$a
+        vector.a.ma.schall[, q] = mean.coefficients + ca$a * 
+            gg$a
     }
     Z <- list()
     coefficients <- list()
     final.lambdas <- list()
     intercept = vector.a.ma.schall[1, ]
     B = B[, -1]
-    vector.a.ma.schall = vector.a.ma.schall[-1, ]
+    vector.a.ma.schall = vector.a.ma.schall[-1, , drop = FALSE]
     for (k in 1:length(design)) {
         final.lambdas[[k]] = lala[k, ]
         partbasis = (sum(nb[0:(k - 1)]) + 1):(sum(nb[0:k]))
@@ -119,19 +157,20 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
             coefficients[[k]] = matrix(NA, nrow = nb[k], ncol = np)
             for (i in 1:np) {
                 Z[[k]][, i] <- B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
-                  i]
+                  i, drop = FALSE]
             }
-            Z[[k]] = Z[[k]][order(x[[k]]), ]
             plot(x[[k]], yy, cex = 0.5, pch = 20, col = "grey42", 
                 xlab = "x", ylab = "y", ylim = range(cbind(yy, 
-                  Z[[k]])))
+                  Z[[k]]), na.rm = TRUE))
             matlines(sort(x[[k]])[seq(1, m, length = min(m, 100))], 
-                Z[[k]][seq(1, m, length = min(m, 100)), ], col = rainbow(np + 
-                  1)[1:11], lty = 1)
-            legend(x = "bottomright", pch = 19, cex = 1, col = rev(rainbow(np + 
-                1)[1:np]), legend = rev(pp), bg = "white", bty = "n")
+                Z[[k]][order(x[[k]])[seq(1, m, length = min(m, 
+                  100))], pp.plot], col = rainbow(np.plot + 1)[1:np.plot], 
+                lty = 1)
+            legend(x = "topright", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp), bg = "white", 
+                bty = "n")
         }
         else if (types[[k]] == "markov") {
             Z[[k]] <- matrix(NA, m, np)
@@ -140,31 +179,32 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
             z = NULL
             for (i in 1:np) {
                 Z[[k]][, i] = B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = Zspathelp[[k]] %*% vector.a.ma.schall[partbasis, 
-                  i]
+                  i, drop = FALSE]
                 z = cbind(z, diag(dim(Zspathelp[[k]])[1]) %*% 
                   Zspathelp[[k]] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i])
+                  i, drop = FALSE] + intercept[i])
             }
             if (class(bnd[[k]]) != "bnd") {
                 plot(seq(0, 1.1 * max(x[[k]]), length = 10), 
-                  seq(0, max(z[, i]), length = 10), type = "n", 
+                  seq(0, max(z[, pp.plot]), length = 10), type = "n", 
                   xlab = "District", ylab = "coefficients")
                 points(rep(as.numeric(attr(bnd[[k]], "regions")), 
-                  times = np), z[, i], col = rainbow(np + 1)[1:np])
-                legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np + 
-                  1)[1:np]), legend = rev(pp), bg = "white", 
-                  bty = "n")
+                  times = np), z[, pp.plot], col = rainbow(np.plot + 
+                  1)[1:np.plot])
+                legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                  1)[1:np.plot]), legend = rev(pp.plot/100), 
+                  bg = "white", bty = "n")
             }
             else {
-                par(mfrow = (c(3, 4)))
+                par(mfrow = (c(row.grid, col.grid)))
                 plot.limits = range(z)
-                for (i in 1:np) {
+                for (i in 1:np.plot) {
                   re = data.frame(cbind(as.numeric(attr(bnd[[k]], 
-                    "regions")), z[, i]))
+                    "regions")), z[, pp.plot[i]]))
                   drawmap(re, bnd[[k]], regionvar = 1, plotvar = 2, 
-                    mar.min = NULL, limits = plot.limits, main = pp[i])
+                    mar.min = NULL, limits = plot.limits, main = pp.plot[i]/100)
                 }
             }
         }
@@ -177,19 +217,22 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
                 times = 50), rep(seq(x.min[2], x.max[2], length = 50), 
                 each = 50))
             B.gitter = base(x.gitter, "2dspline")[[1]]
-            par(mfrow = (c(3, 4)))
+            par(mfrow = (c(row.grid, col.grid)))
             for (i in 1:np) {
-                z <- B.gitter %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
                 Z[[k]][, i] = B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
-                  i]
-                z = t(matrix(z, nrow = 50, ncol = 50))
-                persp(seq(x.min[1], x.max[1], length = 50), seq(x.min[2], 
-                  x.max[2], length = 50), z, ticktype = "detailed", 
-                  phi = 40, zlim = range(yy), col = "lightblue", 
-                  xlab = "X", ylab = "Y", zlab = "Z", main = pp[i])
+                  i, drop = FALSE]
+                if (i %in% pp.plot) {
+                  z <- B.gitter %*% vector.a.ma.schall[partbasis, 
+                    i, drop = FALSE] + intercept[i]
+                  z = t(matrix(z, nrow = 50, ncol = 50))
+                  persp(seq(x.min[1], x.max[1], length = 50), 
+                    seq(x.min[2], x.max[2], length = 50), z, 
+                    ticktype = "detailed", phi = 40, zlim = range(yy), 
+                    col = "lightblue", xlab = "X", ylab = "Y", 
+                    zlab = "Z", main = pp.plot[i]/100)
+                }
             }
         }
         else if (types[[k]] == "radial") {
@@ -200,27 +243,30 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
             x.gitter = cbind(rep(seq(x.min[1], x.max[1], length = 50), 
                 times = 50), rep(seq(x.min[2], x.max[2], length = 50), 
                 each = 50))
-            x[[k]] = x[[k]][order(x[[k]][, 1]), ]
-            knots = x[[k]][seq(1, dim(x[[k]])[1], length = min(50, 
+            x.ord = x[[k]][order(x[[k]][, 1]), ]
+            knots = x.ord[seq(1, dim(x[[k]])[1], length = min(50, 
                 dim(x[[k]])[1])), ]
             B.gitter = matrix(NA, nrow = dim(x.gitter)[1], ncol = dim(knots)[1])
             for (i in 1:dim(x.gitter)[1]) for (j in 1:dim(knots)[1]) {
                 r = sqrt(sum((x.gitter[i, ] - knots[j, ])^2))
                 B.gitter[i, j] = r^2 * log(r)
             }
-            par(mfrow = (c(3, 4)))
+            par(mfrow = (c(row.grid, col.grid)))
             for (i in 1:np) {
-                z <- B.gitter %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
                 Z[[k]][, i] = B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
-                  i]
-                z = t(matrix(z, nrow = 50, ncol = 50))
-                persp(seq(x.min[1], x.max[1], length = 50), seq(x.min[2], 
-                  x.max[2], length = 50), z, ticktype = "detailed", 
-                  phi = 40, zlim = range(yy), col = "lightblue", 
-                  xlab = "X", ylab = "Y", zlab = "Z", main = pp[i])
+                  i, drop = FALSE]
+                if (i %in% pp.plot) {
+                  z <- B.gitter %*% vector.a.ma.schall[partbasis, 
+                    i, drop = FALSE] + intercept[i]
+                  z = t(matrix(z, nrow = 50, ncol = 50))
+                  persp(seq(x.min[1], x.max[1], length = 50), 
+                    seq(x.min[2], x.max[2], length = 50), z, 
+                    ticktype = "detailed", phi = 40, zlim = range(yy), 
+                    col = "lightblue", xlab = "X", ylab = "Y", 
+                    zlab = "Z", main = pp.plot[i]/100)
+                }
             }
         }
         else if (types[[k]] == "krig") {
@@ -231,27 +277,30 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
             x.gitter = cbind(rep(seq(x.min[1], x.max[1], length = 50), 
                 times = 50), rep(seq(x.min[2], x.max[2], length = 50), 
                 each = 50))
-            x[[k]] = x[[k]][order(x[[k]][, 1]), ]
-            knots = x[[k]][seq(1, dim(x[[k]])[1], length = min(50, 
+            x.ord = x[[k]][order(x[[k]][, 1]), ]
+            knots = x.ord[seq(1, dim(x[[k]])[1], length = min(50, 
                 dim(x[[k]])[1])), ]
             B.gitter = matrix(NA, nrow = dim(x.gitter)[1], ncol = dim(knots)[1])
             for (i in 1:dim(x.gitter)[1]) for (j in 1:dim(knots)[1]) {
                 r = sqrt(sum((x.gitter[i, ] - knots[j, ])^2))/krig.phi[[k]]
                 B.gitter[i, j] = exp(-r) * (1 + r)
             }
-            par(mfrow = (c(3, 4)))
+            par(mfrow = (c(row.grid, col.grid)))
             for (i in 1:np) {
-                z <- B.gitter %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
                 Z[[k]][, i] = B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
-                  i]
-                z = t(matrix(z, nrow = 50, ncol = 50))
-                persp(seq(x.min[1], x.max[1], length = 50), seq(x.min[2], 
-                  x.max[2], length = 50), z, ticktype = "detailed", 
-                  phi = 40, zlim = range(yy), col = "lightblue", 
-                  xlab = "X", ylab = "Y", zlab = "Z", main = pp[i])
+                  i, drop = FALSE]
+                if (i %in% pp.plot) {
+                  z <- B.gitter %*% vector.a.ma.schall[partbasis, 
+                    i, drop = FALSE] + intercept[i]
+                  z = t(matrix(z, nrow = 50, ncol = 50))
+                  persp(seq(x.min[1], x.max[1], length = 50), 
+                    seq(x.min[2], x.max[2], length = 50), z, 
+                    ticktype = "detailed", phi = 40, zlim = range(yy), 
+                    col = "lightblue", xlab = "X", ylab = "Y", 
+                    zlab = "Z", main = pp.plot[i]/100)
+                }
             }
         }
         else if (types[[k]] == "random") {
@@ -259,39 +308,62 @@ function (formula, smooth = c("schall", "acv", "none"), lambda = 0.1)
             coefficients[[k]] = matrix(NA, nrow = nb[k], ncol = np)
             for (i in 1:np) {
                 Z[[k]][, i] <- B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
-                  i]
+                  i, drop = FALSE]
             }
             plot(seq(0, 1.1 * max(x[[k]]), length = 10), seq(0, 
                 max(coefficients[[k]] + intercept), length = 10), 
                 type = "n", xlab = "Group", ylab = "coefficients")
-            points(rep(sort(unique(x[[k]])), times = np), coefficients[[k]] + 
-                intercept, col = rainbow(np + 1)[1:np])
-            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np + 
-                1)[1:np]), legend = rev(pp), bg = "white", bty = "n")
+            points(rep(sort(unique(x[[k]])), times = np.plot), 
+                (coefficients[[k]] + intercept)[, pp.plot], col = rainbow(np.plot + 
+                  1)[1:np.plot])
+            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp.plot/100), bg = "white", 
+                bty = "n")
         }
         else if (types[[k]] == "ridge") {
             Z[[k]] <- matrix(NA, m, np)
             coefficients[[k]] = matrix(NA, nrow = nb[k], ncol = np)
             for (i in 1:np) {
                 Z[[k]][, i] <- B[, partbasis] %*% vector.a.ma.schall[partbasis, 
-                  i] + intercept[i]
+                  i, drop = FALSE] + intercept[i]
                 coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
-                  i]
+                  i, drop = FALSE]
             }
             plot(seq(0, 1.1 * dim(x[[k]])[2], length = 10), seq(0, 
                 max(coefficients[[k]] + intercept), length = 10), 
                 type = "n", xlab = "X variables", ylab = "coefficients")
-            points(rep(1:dim(x[[k]])[2], times = np), coefficients[[k]] + 
-                intercept, col = rainbow(np + 1)[1:np])
-            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np + 
-                1)[1:np]), legend = rev(pp), bg = "white", bty = "n")
+            points(rep(1:dim(x[[k]])[2], times = np.plot), (coefficients[[k]] + 
+                intercept)[, pp.plot], col = rainbow(np.plot + 
+                1)[1:np.plot])
+            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp.plot/100), bg = "white", 
+                bty = "n")
+        }
+        else if (types[[k]] == "parametric") {
+            Z[[k]] <- matrix(NA, m, np)
+            coefficients[[k]] = matrix(NA, nrow = nb[k], ncol = np)
+            for (i in 1:np) {
+                Z[[k]][, i] <- B[, partbasis] %*% vector.a.ma.schall[partbasis, 
+                  i, drop = FALSE] + intercept[i]
+                coefficients[[k]][, i] = vector.a.ma.schall[partbasis, 
+                  i, drop = FALSE]
+            }
+            plot(x[[k]], yy, cex = 0.5, pch = 20, col = "grey42", 
+                xlab = "x", ylab = "y", ylim = range(cbind(yy, 
+                  Z[[k]])))
+            matlines(sort(x[[k]]), Z[[k]][order(x[[k]]), pp.plot], 
+                col = rainbow(np.plot + 1)[1:np.plot], lty = 1)
+            legend(x = "bottomright", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp), bg = "white", 
+                bty = "n")
         }
     }
     result = list(lambda = final.lambdas, intercepts = intercept, 
-        coefficients = coefficients, values = Z, response = yy, 
-        covariates = x, formula = formula)
+        coefficients = coefficients, trend.coef = mean.coefficients, 
+        residual.coef = gg$a, asymmetry = cc, values = Z, response = yy, 
+        covariates = x, formula = formula, expectiles = pp)
     class(result) = c("expectreg", "restricted")
     result
 }
