@@ -17,6 +17,8 @@ function (y, B, b, p, cc, DD, lambda, nb)
     lambda = c(0, rep(lambda, times = nb))
     P <- sqrt(lambda) * t(DD) %*% DD
     model <- lsfit(x = BtB + P, y = Bty, intercept = FALSE)
+    sigma = 0.5 * sum((Bty - (BtB + P) %*% model$coef)^2, na.rm = TRUE)/(length(Bty) - 
+        sum(hat(model$qr)[1:length(Bty)]))
     return(list(b = model$coef, hatma = hat(model$qr)[1:length(Bty)], 
-        weight = w))
+        weight = w, sig = sigma))
 }
