@@ -11,11 +11,27 @@ function (x, ...)
     formula = expectreg$formula
     intercept = expectreg$intercepts
     m = length(yy)
-    if (ncol(coefficients[[1]]) != 11) 
-        pp <- seq(0.01, 0.99, by = 0.01)
-    else pp <- c(0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 0.8, 0.9, 0.95, 
-        0.98, 0.99)
+    pp = expectreg$expectiles
     np <- length(pp)
+    if (identical(pp, seq(0.01, 0.99, by = 0.01))) {
+        pp.plot <- c(1, 2, 5, 10, 20, 50, 80, 90, 95, 98, 99)
+        row.grid = 3
+        col.grid = 4
+    }
+    else if (identical(pp, c(0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 
+        0.8, 0.9, 0.95, 0.98, 0.99))) {
+        pp.plot <- 1:length(pp)
+        row.grid = 3
+        col.grid = 4
+    }
+    else {
+        pp.plot <- 1:length(pp)
+        row.grid = floor(sqrt(length(pp)))
+        col.grid = ceiling(sqrt(length(pp)))
+        if (length(pp) > row.grid * col.grid) 
+            row.grid = row.grid + 1
+    }
+    np.plot <- length(pp.plot)
     design <- list()
     types <- list()
     Zspathelp <- list()
@@ -23,7 +39,8 @@ function (x, ...)
     krig.phi = list()
     nb = vector()
     for (i in 1:length(labels(terms(formula)))) {
-        design[[i]] = eval(parse(text = labels(terms(formula))[i]))
+        design[[i]] = eval(parse(text = labels(terms(formula))[i]), 
+            enclos = .GlobalEnv)
         nb[i] = ncol(design[[i]][[1]])
         types[[i]] = design[[i]][[4]]
         bnd[[i]] = design[[i]][[5]]
@@ -31,42 +48,44 @@ function (x, ...)
         krig.phi[[i]] = design[[i]][[7]]
     }
     for (k in 1:length(design)) {
-        partbasis = (sum(nb[0:(k - 1)]) + 1):(sum(nb[0:k]))
         dev.new()
         if (types[[k]] == "pspline") {
+            ZZZ = Z[[k]][order(x[[k]])[seq(1, m, length = min(m, 
+                100))], pp.plot] - mean(Z[[k]]) + mean(yy)
             plot(x[[k]], yy, cex = 0.5, pch = 20, col = "grey42", 
                 xlab = "x", ylab = "y", ylim = range(cbind(yy, 
                   Z[[k]])))
             matlines(sort(x[[k]])[seq(1, m, length = min(m, 100))], 
-                Z[[k]][seq(1, m, length = min(m, 100)), ], col = rainbow(np + 
-                  1)[1:11], lty = 1)
-            legend(x = "bottomright", pch = 19, cex = 1, col = rev(rainbow(np + 
-                1)[1:np]), legend = rev(pp), bg = "white", bty = "n")
+                ZZZ, col = rainbow(np.plot + 1)[1:np.plot], lty = 1)
+            legend(x = "topright", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp), bg = "white", 
+                bty = "n")
         }
         else if (types[[k]] == "markov") {
             z = NULL
             for (i in 1:np) {
                 z = cbind(z, diag(dim(Zspathelp[[k]])[1]) %*% 
-                  coefficients[[k]] + intercept[i])
+                  Zspathelp[[k]] %*% coefficients[[k]] + intercept[i])
             }
             if (class(bnd[[k]]) != "bnd") {
                 plot(seq(0, 1.1 * max(x[[k]]), length = 10), 
-                  seq(0, max(z[, i]), length = 10), type = "n", 
+                  seq(0, max(z[, pp.plot]), length = 10), type = "n", 
                   xlab = "District", ylab = "coefficients")
                 points(rep(as.numeric(attr(bnd[[k]], "regions")), 
-                  times = np), z[, i], col = rainbow(np + 1)[1:np])
-                legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np + 
-                  1)[1:np]), legend = rev(pp), bg = "white", 
-                  bty = "n")
+                  times = np), z[, pp.plot], col = rainbow(np.plot + 
+                  1)[1:np.plot])
+                legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                  1)[1:np.plot]), legend = rev(pp.plot/100), 
+                  bg = "white", bty = "n")
             }
             else {
-                par(mfrow = (c(3, 4)))
+                par(mfrow = (c(row.grid, col.grid)))
                 plot.limits = range(z)
-                for (i in 1:np) {
+                for (i in 1:np.plot) {
                   re = data.frame(cbind(as.numeric(attr(bnd[[k]], 
-                    "regions")), z[, i]))
+                    "regions")), z[, pp.plot[i]]))
                   drawmap(re, bnd[[k]], regionvar = 1, plotvar = 2, 
-                    mar.min = NULL, limits = plot.limits, main = pp[i])
+                    mar.min = NULL, limits = plot.limits, main = pp.plot[i]/100)
                 }
             }
         }
@@ -77,14 +96,18 @@ function (x, ...)
                 times = 50), rep(seq(x.min[2], x.max[2], length = 50), 
                 each = 50))
             B.gitter = base(x.gitter, "2dspline")[[1]]
-            par(mfrow = (c(3, 4)))
+            par(mfrow = (c(row.grid, col.grid)))
             for (i in 1:np) {
-                z <- B.gitter %*% coefficients[[k]][, i] + intercept[i]
-                z = t(matrix(z, nrow = 50, ncol = 50))
-                persp(seq(x.min[1], x.max[1], length = 50), seq(x.min[2], 
-                  x.max[2], length = 50), z, ticktype = "detailed", 
-                  phi = 40, zlim = range(yy), col = "lightblue", 
-                  xlab = "X", ylab = "Y", zlab = "Z", main = pp[i])
+                if (i %in% pp.plot) {
+                  z <- B.gitter %*% coefficients[[k]][, i] + 
+                    intercept[i]
+                  z = t(matrix(z, nrow = 50, ncol = 50))
+                  persp(seq(x.min[1], x.max[1], length = 50), 
+                    seq(x.min[2], x.max[2], length = 50), z, 
+                    ticktype = "detailed", phi = 40, zlim = range(yy), 
+                    col = "lightblue", xlab = "X", ylab = "Y", 
+                    zlab = "Z", main = pp.plot[i]/100)
+                }
             }
         }
         else if (types[[k]] == "radial") {
@@ -101,14 +124,18 @@ function (x, ...)
                 r = sqrt(sum((x.gitter[i, ] - knots[j, ])^2))
                 B.gitter[i, j] = r^2 * log(r)
             }
-            par(mfrow = (c(3, 4)))
+            par(mfrow = (c(row.grid, col.grid)))
             for (i in 1:np) {
-                z <- B.gitter %*% coefficients[[k]][, i] + intercept[i]
-                z = t(matrix(z, nrow = 50, ncol = 50))
-                persp(seq(x.min[1], x.max[1], length = 50), seq(x.min[2], 
-                  x.max[2], length = 50), z, ticktype = "detailed", 
-                  phi = 40, zlim = range(yy), col = "lightblue", 
-                  xlab = "X", ylab = "Y", zlab = "Z", main = pp[i])
+                if (i %in% pp.plot) {
+                  z <- B.gitter %*% coefficients[[k]][, i] + 
+                    intercept[i]
+                  z = t(matrix(z, nrow = 50, ncol = 50))
+                  persp(seq(x.min[1], x.max[1], length = 50), 
+                    seq(x.min[2], x.max[2], length = 50), z, 
+                    ticktype = "detailed", phi = 40, zlim = range(yy), 
+                    col = "lightblue", xlab = "X", ylab = "Y", 
+                    zlab = "Z", main = pp.plot[i]/100)
+                }
             }
         }
         else if (types[[k]] == "krig") {
@@ -125,33 +152,51 @@ function (x, ...)
                 r = sqrt(sum((x.gitter[i, ] - knots[j, ])^2))/krig.phi[[k]]
                 B.gitter[i, j] = exp(-r) * (1 + r)
             }
-            par(mfrow = (c(3, 4)))
+            par(mfrow = (c(row.grid, col.grid)))
             for (i in 1:np) {
-                z <- B.gitter %*% coefficients[[k]][, i] + intercept[i]
-                z = t(matrix(z, nrow = 50, ncol = 50))
-                persp(seq(x.min[1], x.max[1], length = 50), seq(x.min[2], 
-                  x.max[2], length = 50), z, ticktype = "detailed", 
-                  phi = 40, zlim = range(yy), col = "lightblue", 
-                  xlab = "X", ylab = "Y", zlab = "Z", main = pp[i])
+                if (i %in% pp.plot) {
+                  z <- B.gitter %*% coefficients[[k]][, i] + 
+                    intercept[i]
+                  z = t(matrix(z, nrow = 50, ncol = 50))
+                  persp(seq(x.min[1], x.max[1], length = 50), 
+                    seq(x.min[2], x.max[2], length = 50), z, 
+                    ticktype = "detailed", phi = 40, zlim = range(yy), 
+                    col = "lightblue", xlab = "X", ylab = "Y", 
+                    zlab = "Z", main = pp.plot[i]/100)
+                }
             }
         }
         else if (types[[k]] == "random") {
             plot(seq(0, 1.1 * max(x[[k]]), length = 10), seq(0, 
                 max(coefficients[[k]] + intercept), length = 10), 
                 type = "n", xlab = "Group", ylab = "coefficients")
-            points(rep(sort(unique(x[[k]])), times = np), coefficients[[k]] + 
-                intercept, col = rainbow(np + 1)[1:np])
-            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np + 
-                1)[1:np]), legend = rev(pp), bg = "white", bty = "n")
+            points(rep(sort(unique(x[[k]])), times = np.plot), 
+                (coefficients[[k]] + intercept)[, pp.plot], col = rainbow(np.plot + 
+                  1)[1:np.plot])
+            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp.plot/100), bg = "white", 
+                bty = "n")
         }
         else if (types[[k]] == "ridge") {
             plot(seq(0, 1.1 * dim(x[[k]])[2], length = 10), seq(0, 
                 max(coefficients[[k]] + intercept), length = 10), 
                 type = "n", xlab = "X variables", ylab = "coefficients")
-            points(rep(1:dim(x[[k]])[2], times = np), coefficients[[k]] + 
-                intercept, col = rainbow(np + 1)[1:np])
-            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np + 
-                1)[1:np]), legend = rev(pp), bg = "white", bty = "n")
+            points(rep(1:dim(x[[k]])[2], times = np.plot), (coefficients[[k]] + 
+                intercept)[, pp.plot], col = rainbow(np.plot + 
+                1)[1:np.plot])
+            legend(x = "right", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp.plot/100), bg = "white", 
+                bty = "n")
+        }
+        else if (types[[k]] == "parametric") {
+            plot(x[[k]], yy, cex = 0.5, pch = 20, col = "grey42", 
+                xlab = "x", ylab = "y", ylim = range(cbind(yy, 
+                  Z[[k]])))
+            matlines(sort(x[[k]]), Z[[k]][order(x[[k]]), pp.plot], 
+                col = rainbow(np.plot + 1)[1:np.plot], lty = 1)
+            legend(x = "bottomright", pch = 19, cex = 1, col = rev(rainbow(np.plot + 
+                1)[1:np.plot]), legend = rev(pp), bg = "white", 
+                bty = "n")
         }
     }
 }
